@@ -41,7 +41,7 @@
  *
  */
 
-#define DEFAULT_CONF_DIR_PATH "/etc/libblockdev/conf.d/"
+#define DEFAULT_CONF_DIR_PATH "/etc/libblockdev/3/conf.d/"
 
 static GMutex init_lock;
 static gboolean initialized = FALSE;
@@ -55,13 +55,13 @@ typedef void* (*LoadFunc) (const gchar *so_name);
 
 /* KEEP THE ORDERING OF THESE ARRAYS MATCHING THE BDPluginName ENUM! */
 static gchar * default_plugin_so[BD_PLUGIN_UNDEF] = {
-    "libbd_lvm.so.""2", "libbd_btrfs.so.""2",
-    "libbd_swap.so.""2", "libbd_loop.so.""2",
-    "libbd_crypto.so.""2", "libbd_mpath.so.""2",
-    "libbd_dm.so.""2", "libbd_mdraid.so.""2",
-    "libbd_s390.so.""2", "libbd_part.so.""2",
-    "libbd_fs.so.""2", "libbd_nvdimm.so.""2",
-    "libbd_nvme.so.""2"
+    "libbd_lvm.so.3", "libbd_btrfs.so.3",
+    "libbd_swap.so.3", "libbd_loop.so.3",
+    "libbd_crypto.so.3", "libbd_mpath.so.3",
+    "libbd_dm.so.3", "libbd_mdraid.so.3",
+    "libbd_s390.so.3", "libbd_part.so.3",
+    "libbd_fs.so.3", "libbd_nvdimm.so.3",
+    "libbd_nvme.so.3"
 };
 static BDPluginStatus plugins[BD_PLUGIN_UNDEF] = {
     {{BD_PLUGIN_LVM, NULL}, NULL},
