@@ -536,7 +536,7 @@ GType  bd_nvme_sanitize_log_get_type () {
     return type;
 }
 
-static const gchar * bd_nvme_self_test_result_to_string_stub (BDNVMESelfTestResult result, GError **error) {
+static const gchar * bd_nvme_self_test_result_to_string_stub (BDNVMESelfTestResult result G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_self_test_result_to_string' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_self_test_result_to_string' called, but not implemented!");
@@ -558,7 +558,7 @@ const gchar * bd_nvme_self_test_result_to_string (BDNVMESelfTestResult result, G
 }
 
 
-static BDNVMEControllerInfo * bd_nvme_get_controller_info_stub (const gchar *device, GError **error) {
+static BDNVMEControllerInfo * bd_nvme_get_controller_info_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_get_controller_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_get_controller_info' called, but not implemented!");
@@ -584,7 +584,7 @@ BDNVMEControllerInfo * bd_nvme_get_controller_info (const gchar *device, GError
 }
 
 
-static BDNVMENamespaceInfo * bd_nvme_get_namespace_info_stub (const gchar *device, GError **error) {
+static BDNVMENamespaceInfo * bd_nvme_get_namespace_info_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_get_namespace_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_get_namespace_info' called, but not implemented!");
@@ -610,7 +610,7 @@ BDNVMENamespaceInfo * bd_nvme_get_namespace_info (const gchar *device, GError **
 }
 
 
-static BDNVMESmartLog * bd_nvme_get_smart_log_stub (const gchar *device, GError **error) {
+static BDNVMESmartLog * bd_nvme_get_smart_log_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_get_smart_log' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_get_smart_log' called, but not implemented!");
@@ -636,7 +636,7 @@ BDNVMESmartLog * bd_nvme_get_smart_log (const gchar *device, GError **error) {
 }
 
 
-static BDNVMEErrorLogEntry ** bd_nvme_get_error_log_entries_stub (const gchar *device, GError **error) {
+static BDNVMEErrorLogEntry ** bd_nvme_get_error_log_entries_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_get_error_log_entries' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_get_error_log_entries' called, but not implemented!");
@@ -668,7 +668,7 @@ BDNVMEErrorLogEntry ** bd_nvme_get_error_log_entries (const gchar *device, GErro
 }
 
 
-static BDNVMESelfTestLog * bd_nvme_get_self_test_log_stub (const gchar *device, GError **error) {
+static BDNVMESelfTestLog * bd_nvme_get_self_test_log_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_get_self_test_log' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_get_self_test_log' called, but not implemented!");
@@ -695,7 +695,7 @@ BDNVMESelfTestLog * bd_nvme_get_self_test_log (const gchar *device, GError **err
 }
 
 
-static BDNVMESanitizeLog * bd_nvme_get_sanitize_log_stub (const gchar *device, GError **error) {
+static BDNVMESanitizeLog * bd_nvme_get_sanitize_log_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_get_sanitize_log' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_get_sanitize_log' called, but not implemented!");
@@ -725,7 +725,7 @@ BDNVMESanitizeLog * bd_nvme_get_sanitize_log (const gchar *device, GError **erro
 }
 
 
-static gboolean  bd_nvme_device_self_test_stub (const gchar *device, BDNVMESelfTestAction action, GError **error) {
+static gboolean  bd_nvme_device_self_test_stub (const gchar *device G_GNUC_UNUSED, BDNVMESelfTestAction action G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_device_self_test' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_device_self_test' called, but not implemented!");
@@ -758,7 +758,7 @@ gboolean  bd_nvme_device_self_test (const gchar *device, BDNVMESelfTestAction ac
 }
 
 
-static gboolean  bd_nvme_format_stub (const gchar *device, guint16 lba_data_size, guint16 metadata_size, BDNVMEFormatSecureErase secure_erase, GError **error) {
+static gboolean  bd_nvme_format_stub (const gchar *device G_GNUC_UNUSED, guint16 lba_data_size G_GNUC_UNUSED, guint16 metadata_size G_GNUC_UNUSED, BDNVMEFormatSecureErase secure_erase G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_format' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_format' called, but not implemented!");
@@ -802,7 +802,7 @@ gboolean  bd_nvme_format (const gchar *device, guint16 lba_data_size, guint16 me
 }
 
 
-static gboolean  bd_nvme_sanitize_stub (const gchar *device, BDNVMESanitizeAction action, gboolean no_dealloc, gint overwrite_pass_count, guint32 overwrite_pattern, gboolean overwrite_invert_pattern, GError **error) {
+static gboolean  bd_nvme_sanitize_stub (const gchar *device G_GNUC_UNUSED, BDNVMESanitizeAction action G_GNUC_UNUSED, gboolean no_dealloc G_GNUC_UNUSED, gint overwrite_pass_count G_GNUC_UNUSED, guint32 overwrite_pattern G_GNUC_UNUSED, gboolean overwrite_invert_pattern G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_sanitize' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_sanitize' called, but not implemented!");
@@ -936,7 +936,7 @@ gchar * bd_nvme_get_host_id (GError **error) {
 }
 
 
-static gboolean  bd_nvme_set_host_nqn_stub (const gchar *host_nqn, GError **error) {
+static gboolean  bd_nvme_set_host_nqn_stub (const gchar *host_nqn G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_set_host_nqn' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_set_host_nqn' called, but not implemented!");
@@ -962,7 +962,7 @@ gboolean  bd_nvme_set_host_nqn (const gchar *host_nqn, GError **error) {
 }
 
 
-static gboolean  bd_nvme_set_host_id_stub (const gchar *host_id, GError **error) {
+static gboolean  bd_nvme_set_host_id_stub (const gchar *host_id G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_set_host_id' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_set_host_id' called, but not implemented!");
@@ -988,7 +988,7 @@ gboolean  bd_nvme_set_host_id (const gchar *host_id, GError **error) {
 }
 
 
-static gboolean  bd_nvme_connect_stub (const gchar *subsysnqn, const gchar *transport, const gchar *transport_addr, const gchar *transport_svcid, const gchar *host_traddr, const gchar *host_iface, const gchar *host_nqn, const gchar *host_id, const BDExtraArg **extra, GError **error) {
+static gboolean  bd_nvme_connect_stub (const gchar *subsysnqn G_GNUC_UNUSED, const gchar *transport G_GNUC_UNUSED, const gchar *transport_addr G_GNUC_UNUSED, const gchar *transport_svcid G_GNUC_UNUSED, const gchar *host_traddr G_GNUC_UNUSED, const gchar *host_iface G_GNUC_UNUSED, const gchar *host_nqn G_GNUC_UNUSED, const gchar *host_id G_GNUC_UNUSED, const BDExtraArg **extra G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_connect' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_connect' called, but not implemented!");
@@ -1066,7 +1066,7 @@ gboolean  bd_nvme_connect (const gchar *subsysnqn, const gchar *transport, const
 }
 
 
-static gboolean  bd_nvme_disconnect_stub (const gchar *subsysnqn, GError **error) {
+static gboolean  bd_nvme_disconnect_stub (const gchar *subsysnqn G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_disconnect' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_disconnect' called, but not implemented!");
@@ -1094,7 +1094,7 @@ gboolean  bd_nvme_disconnect (const gchar *subsysnqn, GError **error) {
 }
 
 
-static gboolean  bd_nvme_disconnect_by_path_stub (const gchar *path, GError **error) {
+static gboolean  bd_nvme_disconnect_by_path_stub (const gchar *path G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_disconnect_by_path' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_disconnect_by_path' called, but not implemented!");
@@ -1121,7 +1121,7 @@ gboolean  bd_nvme_disconnect_by_path (const gchar *path, GError **error) {
 }
 
 
-static gchar ** bd_nvme_find_ctrls_for_ns_stub (const gchar *ns_sysfs_path, const gchar *subsysnqn, const gchar *host_nqn, const gchar *host_id, GError **error) {
+static gchar ** bd_nvme_find_ctrls_for_ns_stub (const gchar *ns_sysfs_path G_GNUC_UNUSED, const gchar *subsysnqn G_GNUC_UNUSED, const gchar *host_nqn G_GNUC_UNUSED, const gchar *host_id G_GNUC_UNUSED, GError **error) {
     g_critical ("The function 'bd_nvme_find_ctrls_for_ns' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_nvme_find_ctrls_for_ns' called, but not implemented!");
