@@ -2,7 +2,7 @@ import os
 import unittest
 import re
 import overrides_hack
-from utils import fake_path
+from utils import fake_path, TestTags, tag_test
 
 from gi.repository import GLib, BlockDev
 
@@ -13,10 +13,8 @@ class LibraryOpsTestCase(unittest.TestCase):
     # all plugins except for 'btrfs', 'fs' and 'mpath' -- these don't have all
     # the dependencies on CentOS/Debian and we don't need them for this test
     requested_plugins = BlockDev.plugin_specs_from_names(("crypto", "dm",
-                                                          "kbd", "loop", "lvm",
-                                                          "mdraid", "part", "swap"))
-
-    orig_config_dir = ""
+                                                          "loop", "mdraid",
+                                                          "part", "swap"))
 
     @classmethod
     def setUpClass(cls):
@@ -25,224 +23,6 @@ class LibraryOpsTestCase(unittest.TestCase):
         else:
             BlockDev.reinit(cls.requested_plugins, True, None)
 
-    def setUp(self):
-        self.orig_config_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR", "")
-        self.addCleanup(self._clean_up)
-
-    def _clean_up(self):
-        # change the sources back and recompile
-        os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = self.orig_config_dir
-
-        # try to get everything back to normal by (re)loading all plugins
-        BlockDev.reinit(self.requested_plugins, True, None)
-
-    # recompiles the LVM plugin
-    @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
-    def test_reload(self):
-        """Verify that reloading plugins works as expected"""
-
-        # max LV size should be something sane (not 1024 bytes)
-        orig_max_size = BlockDev.lvm_get_max_lv_size()
-        self.assertNotEqual(orig_max_size, 1024)
-
-        # change the sources and recompile
-        os.system("sed -ri 's?BD_LVM_MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # library should successfully reinitialize without reloading plugins
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, False, None))
-
-        # LVM plugin not reloaded, max LV size should be the same
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-        # library should successfully reinitialize reloading plugins
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        # LVM plugin reloaded, max LV size should be 1024 bytes
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
-
-        # change the sources back and recompile
-        os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # library should successfully reinitialize reloading original plugins
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-    # recompiles the LVM plugin
-    @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
-    def test_force_plugin(self):
-        """Verify that forcing plugin to be used works as expected"""
-
-        # library should be successfully initialized
-        self.assertTrue(BlockDev.is_initialized())
-
-        # init() called twice, should give a warning and return False
-        self.assertFalse(BlockDev.init(self.requested_plugins, None))
-
-        # max LV size should be something sane (not 1024 bytes)
-        orig_max_size = BlockDev.lvm_get_max_lv_size()
-        self.assertNotEqual(orig_max_size, 1024)
-
-        # change the sources and recompile
-        os.system("sed -ri 's?BD_LVM_MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # proclaim the new build a different plugin
-        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so")
-
-        # change the sources back and recompile
-        os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # force the new plugin to be used
-        ps = BlockDev.PluginSpec()
-        ps.name = BlockDev.Plugin.LVM
-        ps.so_name = "libbd_lvm2.so"
-        self.assertTrue(BlockDev.reinit([ps], True, None))
-
-        # new LVM plugin loaded, max LV size should be 1024 bytes
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
-
-        # clean after ourselves
-        os.system ("rm -f src/plugins/.libs/libbd_lvm2.so")
-
-        # force the old plugin to be used
-        ps = BlockDev.PluginSpec()
-        ps.name = BlockDev.Plugin.LVM
-        ps.so_name = "libbd_lvm.so"
-        self.assertTrue(BlockDev.reinit([ps], True, None))
-
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-    # recompiles the LVM plugin
-    @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
-    def test_plugin_priority(self):
-        """Verify that preferring plugin to be used works as expected"""
-
-        # library should be successfully initialized
-        self.assertTrue(BlockDev.is_initialized())
-
-        # max LV size should be something sane (not 1024 bytes)
-        orig_max_size = BlockDev.lvm_get_max_lv_size()
-        self.assertNotEqual(orig_max_size, 1024)
-
-        # change the sources and recompile
-        os.system("sed -ri 's?BD_LVM_MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # proclaim the new build a different plugin
-        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.2")
-
-        # change the sources back and recompile
-        os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # now reinit the library with the config preferring the new build
-        orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
-        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_prio_conf.d"
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        # new LVM plugin loaded, max LV size should be 1024 bytes
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.2")
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
-
-        # reinit with the original config
-        if orig_conf_dir:
-            os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
-        else:
-            del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-        # now reinit the library with the another config preferring the new
-        # build
-        orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
-        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_multi_conf.d"
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        # new LVM plugin loaded, max LV size should be 1024 bytes
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.2")
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
-
-        # reinit with the original config
-        if orig_conf_dir:
-            os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
-        else:
-            del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-        # clean after ourselves
-        os.system ("rm -f src/plugins/.libs/libbd_lvm2.so")
-
-    # recompiles the LVM plugin
-    @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
-    def test_plugin_fallback(self):
-        """Verify that fallback when loading plugins works as expected"""
-
-        # library should be successfully initialized
-        self.assertTrue(BlockDev.is_initialized())
-
-        # max LV size should be something sane (not 1024 bytes)
-        orig_max_size = BlockDev.lvm_get_max_lv_size()
-        self.assertNotEqual(orig_max_size, 1024)
-
-        # change the sources and recompile
-        os.system("sed -ri 's?gboolean bd_lvm_check_deps \(void\) \{?gboolean bd_lvm_check_deps (void) { return FALSE;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # proclaim the new build a different plugin
-        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.2")
-        self.addCleanup(os.system, "rm -f src/plugins/.libs/libbd_lvm2.so")
-
-        # change the sources back and recompile
-        os.system("sed -ri 's?gboolean bd_lvm_check_deps \(void\) \{ return FALSE;//test-change?gboolean bd_lvm_check_deps (void) {?' src/plugins/lvm.c > /dev/null")
-        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
-
-        # now reinit the library with the config preferring the new build
-        orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
-        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_prio_conf.d"
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        # the original plugin should be loaded because the new one should fail
-        # to load (due to check() returning FALSE)
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.2")
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-        # reinit with the original config
-        if orig_conf_dir:
-            os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
-        else:
-            del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-        # now reinit the library with the another config preferring the new
-        # build
-        orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
-        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_multi_conf.d"
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        # the original plugin should be loaded because the new one should fail
-        # to load (due to check() returning FALSE)
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.2")
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
-        # reinit with the original config
-        if orig_conf_dir:
-            os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
-        else:
-            del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
-
     def my_log_func(self, level, msg):
         # not much to verify here
         self.assertTrue(isinstance(level, int))
@@ -250,11 +30,15 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         self.log += msg + "\n"
 
+    @tag_test(TestTags.CORE)
     def test_logging_setup(self):
         """Verify that setting up logging works as expected"""
 
         self.assertTrue(BlockDev.reinit(self.requested_plugins, False, self.my_log_func))
 
+        # we are checking for info log messages and default level is warning
+        BlockDev.utils_set_log_level(BlockDev.UTILS_LOG_INFO)
+
         succ = BlockDev.utils_exec_and_report_error(["true"])
         self.assertTrue(succ)
 
@@ -280,6 +64,7 @@ class LibraryOpsTestCase(unittest.TestCase):
         self.assertIn("stderr[%s]:" % task_id2, self.log)
         self.assertIn("...done [%s] (exit code: 0)" % task_id2, self.log)
 
+    @tag_test(TestTags.CORE)
     def test_require_plugins(self):
         """Verify that loading only required plugins works as expected"""
 
@@ -290,11 +75,12 @@ class LibraryOpsTestCase(unittest.TestCase):
         self.assertEqual(BlockDev.get_available_plugin_names(), ["swap"])
         self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
+    @tag_test(TestTags.CORE)
     def test_not_implemented(self):
         """Verify that unloaded/unimplemented functions report errors"""
 
         # should be loaded and working
-        self.assertTrue(BlockDev.lvm_get_max_lv_size() > 0)
+        self.assertTrue(BlockDev.md_canonicalize_uuid("3386ff85:f5012621:4a435f06:1eb47236"))
 
         ps = BlockDev.PluginSpec()
         ps.name = BlockDev.Plugin.SWAP
@@ -304,12 +90,12 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # no longer loaded
         with self.assertRaises(GLib.GError):
-            BlockDev.lvm_get_max_lv_size()
+            BlockDev.md_canonicalize_uuid("3386ff85:f5012621:4a435f06:1eb47236")
 
         self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # loaded again
-        self.assertTrue(BlockDev.lvm_get_max_lv_size() > 0)
+        self.assertTrue(BlockDev.md_canonicalize_uuid("3386ff85:f5012621:4a435f06:1eb47236"))
 
     def test_ensure_init(self):
         """Verify that ensure_init just returns when already initialized"""
@@ -320,18 +106,18 @@ class LibraryOpsTestCase(unittest.TestCase):
         self.assertEqual(avail_plugs, BlockDev.get_available_plugin_names())
 
         # reinit with a subset of plugins
-        plugins = BlockDev.plugin_specs_from_names(["swap", "lvm"])
+        plugins = BlockDev.plugin_specs_from_names(["swap", "part"])
         self.assertTrue(BlockDev.reinit(plugins, True, None))
-        self.assertEqual(set(BlockDev.get_available_plugin_names()), set(["swap", "lvm"]))
+        self.assertEqual(set(BlockDev.get_available_plugin_names()), set(["swap", "part"]))
 
         # ensure_init with the same subset -> nothing should change
         self.assertTrue(BlockDev.ensure_init(plugins, None))
-        self.assertEqual(set(BlockDev.get_available_plugin_names()), set(["swap", "lvm"]))
+        self.assertEqual(set(BlockDev.get_available_plugin_names()), set(["swap", "part"]))
 
         # ensure_init with more plugins -> extra plugins should be loaded
-        plugins = BlockDev.plugin_specs_from_names(["swap", "lvm", "crypto"])
+        plugins = BlockDev.plugin_specs_from_names(["swap", "part", "crypto"])
         self.assertTrue(BlockDev.ensure_init(plugins, None))
-        self.assertEqual(set(BlockDev.get_available_plugin_names()), set(["swap", "lvm", "crypto"]))
+        self.assertEqual(set(BlockDev.get_available_plugin_names()), set(["swap", "part", "crypto"]))
 
         # reinit to unload all plugins
         self.assertTrue(BlockDev.reinit([], True, None))
@@ -339,30 +125,7 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # ensure_init to load all plugins back
         self.assertTrue(BlockDev.ensure_init(self.requested_plugins, None))
-        self.assertGreaterEqual(len(BlockDev.get_available_plugin_names()), 8)
-
-    def test_try_reinit(self):
-        """Verify that try_reinit() works as expected"""
-
-        # try reinitializing with only some utilities being available and thus
-        # only some plugins able to load
-        with fake_path("tests/lib_missing_utils", keep_utils=["swapon", "swapoff", "mkswap", "lvm",
-                                                              "thin_metadata_size", "swaplabel"]):
-            succ, loaded = BlockDev.try_reinit(self.requested_plugins, True, None)
-            self.assertFalse(succ)
-            for plug_name in ("swap", "lvm", "crypto"):
-                self.assertIn(plug_name, loaded)
-
-        # reset back to all plugins
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-
-        # now the same with a subset of plugins requested
-        plugins = BlockDev.plugin_specs_from_names(["lvm", "swap", "crypto"])
-        with fake_path("tests/lib_missing_utils", keep_utils=["swapon", "swapoff", "mkswap", "lvm",
-                                                              "thin_metadata_size", "swaplabel"]):
-            succ, loaded = BlockDev.try_reinit(plugins, True, None)
-            self.assertTrue(succ)
-            self.assertEqual(set(loaded), set(["swap", "lvm", "crypto"]))
+        self.assertGreaterEqual(len(BlockDev.get_available_plugin_names()), 6)
 
     def test_non_en_init(self):
         """Verify that the library initializes with lang different from en_US"""
@@ -376,17 +139,176 @@ class LibraryOpsTestCase(unittest.TestCase):
             del os.environ["LANG"]
         self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
-    def test_dep_checks_disabled(self):
-        """Verify that disabling runtime dep checks works"""
 
-        with fake_path(all_but="mkswap"):
-            # should fail because of 'mkswap' missing
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
+class PluginsTestCase(unittest.TestCase):
+    # only LVM plugin for this test
+    requested_plugins = BlockDev.plugin_specs_from_names(("lvm",))
 
-        os.environ["LIBBLOCKDEV_SKIP_DEP_CHECKS"] = ""
-        self.addCleanup(os.environ.pop, "LIBBLOCKDEV_SKIP_DEP_CHECKS")
+    orig_config_dir = ""
 
-        with fake_path(all_but="mkswap"):
-            # should load just fine, skipping the runtime dep checks
-            self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+    @classmethod
+    def setUpClass(cls):
+        if not BlockDev.is_initialized():
+            BlockDev.init(cls.requested_plugins, None)
+        else:
+            BlockDev.reinit(cls.requested_plugins, True, None)
+
+        try:
+            cls.devices_avail = BlockDev.lvm_is_tech_avail(BlockDev.LVMTech.DEVICES, 0)
+        except:
+            cls.devices_avail = False
+
+    def setUp(self):
+        self.orig_config_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR", "")
+        self.addCleanup(self._clean_up)
+
+    def _clean_up(self):
+        # change the sources back and recompile
+        os.system("sed -ri 's?1024;//test-change?MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = self.orig_config_dir
+
+        # try to get everything back to normal by (re)loading all plugins
+        BlockDev.reinit(self.requested_plugins, True, None)
+
+    # recompiles the LVM plugin
+    @tag_test(TestTags.SLOW, TestTags.CORE, TestTags.SOURCEONLY)
+    def test_reload(self):
+        """Verify that reloading plugins works as expected"""
+
+        # max LV size should be something sane (not 1024 bytes)
+        orig_max_size = BlockDev.lvm_get_max_lv_size()
+        self.assertNotEqual(orig_max_size, 1024)
+
+        # change the sources and recompile
+        os.system("sed -ri 's?MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        # library should successfully reinitialize without reloading plugins
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, False, None))
+
+        # LVM plugin not reloaded, max LV size should be the same
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
+
+        # library should successfully reinitialize reloading plugins
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+        # LVM plugin reloaded, max LV size should be 1024 bytes
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
+
+        # change the sources back and recompile
+        os.system("sed -ri 's?1024;//test-change?MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        # library should successfully reinitialize reloading original plugins
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+    # recompiles the LVM plugin
+    @tag_test(TestTags.SLOW, TestTags.SOURCEONLY)
+    def test_force_plugin(self):
+        """Verify that forcing plugin to be used works as expected"""
+
+        # library should be successfully initialized
+        self.assertTrue(BlockDev.is_initialized())
+
+        # init() called twice, should give a warning and return False
+        self.assertFalse(BlockDev.init(self.requested_plugins, None))
+
+        # max LV size should be something sane (not 1024 bytes)
+        orig_max_size = BlockDev.lvm_get_max_lv_size()
+        self.assertNotEqual(orig_max_size, 1024)
+
+        # change the sources and recompile
+        os.system("sed -ri 's?MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        # proclaim the new build a different plugin
+        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so")
+
+        # change the sources back and recompile
+        os.system("sed -ri 's?1024;//test-change?MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        # force the new plugin to be used
+        ps = BlockDev.PluginSpec()
+        ps.name = BlockDev.Plugin.LVM
+        ps.so_name = "libbd_lvm2.so"
+        self.assertTrue(BlockDev.reinit([ps], True, None))
+
+        # new LVM plugin loaded, max LV size should be 1024 bytes
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
+
+        # clean after ourselves
+        os.system ("rm -f src/plugins/.libs/libbd_lvm2.so")
+
+        # force the old plugin to be used
+        ps = BlockDev.PluginSpec()
+        ps.name = BlockDev.Plugin.LVM
+        ps.so_name = "libbd_lvm.so"
+        self.assertTrue(BlockDev.reinit([ps], True, None))
+
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
+
+    # recompiles the LVM plugin
+    @tag_test(TestTags.SLOW, TestTags.SOURCEONLY)
+    def test_plugin_priority(self):
+        """Verify that preferring plugin to be used works as expected"""
+
+        # library should be successfully initialized
+        self.assertTrue(BlockDev.is_initialized())
+
+        # max LV size should be something sane (not 1024 bytes)
+        orig_max_size = BlockDev.lvm_get_max_lv_size()
+        self.assertNotEqual(orig_max_size, 1024)
+
+        # change the sources and recompile
+        os.system("sed -ri 's?MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        # proclaim the new build a different plugin
+        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.3")
+
+        # change the sources back and recompile
+        os.system("sed -ri 's?1024;//test-change?MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
+
+        # now reinit the library with the config preferring the new build
+        orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
+        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/test_configs/plugin_prio_conf.d"
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+        # new LVM plugin loaded, max LV size should be 1024 bytes
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.3")
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
+
+        # reinit with the original config
+        if orig_conf_dir:
+            os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
+        else:
+            del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
+
+        # now reinit the library with the another config preferring the new
+        # build
+        orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
+        os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/test_configs/plugin_multi_conf.d"
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+        # new LVM plugin loaded, max LV size should be 1024 bytes
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.3")
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
+
+        # reinit with the original config
+        if orig_conf_dir:
+            os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
+        else:
+            del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+        self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
+
+        # clean after ourselves
+        os.system ("rm -f src/plugins/.libs/libbd_lvm2.so")
