@@ -3,19 +3,13 @@ import unittest
 import os
 import math
 import overrides_hack
-import six
 import re
 import shutil
-import subprocess
 import time
 from contextlib import contextmanager
 from packaging.version import Version
 
-from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, fake_utils, fake_path, TestTags, tag_test, run_command
-
-import gi
-gi.require_version("BlockDev", "2.0")
-
+from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, fake_utils, fake_path, TestTags, tag_test, run_command, read_file
 from gi.repository import BlockDev, GLib
 
 
@@ -40,7 +34,7 @@ class LVMTestCase(unittest.TestCase):
     def setUpClass(cls):
         ps = BlockDev.PluginSpec()
         ps.name = BlockDev.Plugin.LVM
-        ps.so_name = "libbd_lvm.so.2"
+        ps.so_name = "libbd_lvm.so.3"
         cls.requested_plugins = [ps]
 
         if not BlockDev.is_initialized():
@@ -48,6 +42,11 @@ class LVMTestCase(unittest.TestCase):
         else:
             BlockDev.reinit(cls.requested_plugins, True, None)
 
+        try:
+            cls.devices_avail = BlockDev.lvm_is_tech_avail(BlockDev.LVMTech.DEVICES, 0)
+        except:
+            cls.devices_avail = False
+
     @classmethod
     def _get_lvm_version(cls):
         _ret, out, _err = run_command("lvm version")
@@ -56,12 +55,37 @@ class LVMTestCase(unittest.TestCase):
             raise RuntimeError("Failed to determine LVM version from: %s" % out)
         return Version(m.groups()[0])
 
+    @classmethod
+    def _get_lvm_segtypes(cls):
+        _ret, out, _err = run_command("lvm segtypes")
+        return out
+
 
 class LvmNoDevTestCase(LVMTestCase):
     def __init__(self, *args, **kwargs):
         super(LvmNoDevTestCase, self).__init__(*args, **kwargs)
+
+    def setUp(self):
         self._log = ""
 
+    @classmethod
+    def setUpClass(cls):
+        # we are checking for info log messages and default level is warning
+        BlockDev.utils_set_log_level(BlockDev.UTILS_LOG_INFO)
+
+        super(LvmNoDevTestCase, cls).setUpClass()
+
+    @classmethod
+    def tearDownClass(cls):
+        # reset back to default
+        BlockDev.utils_set_log_level(BlockDev.UTILS_LOG_WARNING)
+
+        super(LvmNoDevTestCase, cls).tearDownClass()
+
+    @tag_test(TestTags.NOSTORAGE)
+    def test_plugin_version(self):
+       self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.3")
+
     @tag_test(TestTags.NOSTORAGE)
     def test_is_supported_pe_size(self):
         """Verify that lvm_is_supported_pe_size works as expected"""
@@ -168,8 +192,9 @@ class LvmNoDevTestCase(LVMTestCase):
         self.assertEqual(BlockDev.lvm_get_thpool_meta_size(1 * 1024**4, 128 * 1024),
                          512 * 1024**2)
 
+        # min metadata size is 4 MiB
         self.assertEqual(BlockDev.lvm_get_thpool_meta_size(100 * 1024**2, 128 * 1024),
-                         BlockDev.LVM_MIN_THPOOL_MD_SIZE)
+                         4 * 1024**2)
 
     @tag_test(TestTags.NOSTORAGE)
     def test_is_valid_thpool_md_size(self):
@@ -233,6 +258,7 @@ class LvmNoDevTestCase(LVMTestCase):
 
         # set something sane and check it's really used
         succ = BlockDev.lvm_set_global_config("backup {backup=0 archive=0}")
+        self.assertTrue(succ)
         BlockDev.lvm_lvs(None)
         self.assertIn("--config=backup {backup=0 archive=0}", self._log)
 
@@ -240,6 +266,45 @@ class LvmNoDevTestCase(LVMTestCase):
         succ = BlockDev.lvm_set_global_config(None)
         self.assertTrue(succ)
 
+    @tag_test(TestTags.NOSTORAGE)
+    def test_get_set_global_devices_filter(self):
+        """Verify that getting and setting LVM devices filter works as expected"""
+        if not self.devices_avail:
+            self.skipTest("skipping LVM devices filter test: not supported")
+
+        # setup logging
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, False, self._store_log))
+
+        # no global config set initially
+        self.assertListEqual(BlockDev.lvm_get_devices_filter(), [])
+
+        # set and try to get back
+        succ = BlockDev.lvm_set_devices_filter(["/dev/sda"])
+        self.assertTrue(succ)
+        self.assertListEqual(BlockDev.lvm_get_devices_filter(), ["/dev/sda"])
+
+        # reset and try to get back
+        succ = BlockDev.lvm_set_devices_filter(None)
+        self.assertTrue(succ)
+        self.assertListEqual(BlockDev.lvm_get_devices_filter(), [])
+
+        # set twice and try to get back twice
+        succ = BlockDev.lvm_set_devices_filter(["/dev/sda"])
+        self.assertTrue(succ)
+        succ = BlockDev.lvm_set_devices_filter(["/dev/sdb"])
+        self.assertTrue(succ)
+        self.assertEqual(BlockDev.lvm_get_devices_filter(), ["/dev/sdb"])
+
+        # set something sane and check it's really used
+        succ = BlockDev.lvm_set_devices_filter(["/dev/sdb", "/dev/sdc"])
+        self.assertTrue(succ)
+        BlockDev.lvm_lvs(None)
+        self.assertIn("--devices=/dev/sdb,/dev/sdc", self._log)
+
+        # reset back to default
+        succ = BlockDev.lvm_set_devices_filter(None)
+        self.assertTrue(succ)
+
     @tag_test(TestTags.NOSTORAGE)
     def test_cache_get_default_md_size(self):
         """Verify that default cache metadata size is calculated properly"""
@@ -268,24 +333,6 @@ class LvmPVonlyTestCase(LVMTestCase):
 
     _sparse_size = 1024**3
 
-    @classmethod
-    def setUpClass(cls):
-        super().setUpClass()
-
-        try:
-            BlockDev.lvm_set_global_config("devices {use_devicesfile = 0}")
-        except:
-            pass
-
-    @classmethod
-    def tearDownClass(cls):
-        super().tearDownClass()
-
-        try:
-            BlockDev.lvm_set_global_config("")
-        except:
-            pass
-
     # :TODO:
     #     * test pvmove (must create two PVs, a VG, a VG and some data in it
     #       first)
@@ -294,6 +341,7 @@ class LvmPVonlyTestCase(LVMTestCase):
         self.addCleanup(self._clean_up)
         self.dev_file = create_sparse_tempfile("lvm_test", self._sparse_size)
         self.dev_file2 = create_sparse_tempfile("lvm_test", self._sparse_size)
+        self.dev_file3 = create_sparse_tempfile("lvm_test", self._sparse_size)
         try:
             self.loop_dev = create_lio_device(self.dev_file)
         except RuntimeError as e:
@@ -302,6 +350,10 @@ class LvmPVonlyTestCase(LVMTestCase):
             self.loop_dev2 = create_lio_device(self.dev_file2)
         except RuntimeError as e:
             raise RuntimeError("Failed to setup loop device for testing: %s" % e)
+        try:
+            self.loop_dev3 = create_lio_device(self.dev_file3)
+        except RuntimeError as e:
+            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
 
     def _clean_up(self):
         try:
@@ -314,6 +366,11 @@ class LvmPVonlyTestCase(LVMTestCase):
         except:
             pass
 
+        try:
+            BlockDev.lvm_pvremove(self.loop_dev3, None)
+        except:
+            pass
+
         try:
             delete_lio_device(self.loop_dev)
         except RuntimeError:
@@ -328,6 +385,13 @@ class LvmPVonlyTestCase(LVMTestCase):
             pass
         os.unlink(self.dev_file2)
 
+        try:
+            delete_lio_device(self.loop_dev3)
+        except RuntimeError:
+            # just move on, we can do no better here
+            pass
+        os.unlink(self.dev_file3)
+
 class LvmTestPVcreateRemove(LvmPVonlyTestCase):
     @tag_test(TestTags.CORE)
     def test_pvcreate_and_pvremove(self):
@@ -372,15 +436,9 @@ class LvmTestPVresize(LvmPVonlyTestCase):
         succ = BlockDev.lvm_pvresize(self.loop_dev, 200 * 1024**2, None)
         self.assertTrue(succ)
 
-        info = BlockDev.lvm_pvinfo(self.loop_dev)
-        self.assertEqual(info.pv_size, 200 * 1024**2)
-
         succ = BlockDev.lvm_pvresize(self.loop_dev, 200 * 1024**3, None)
         self.assertTrue(succ)
 
-        info = BlockDev.lvm_pvinfo(self.loop_dev)
-        self.assertEqual(info.pv_size, 200 * 1024**3)
-
 class LvmTestPVscan(LvmPVonlyTestCase):
     def test_pvscan(self):
         """Verify that pvscan runs without issues with cache or without"""
@@ -417,7 +475,7 @@ class LvmTestPVs(LvmPVonlyTestCase):
         self.assertTrue(succ)
 
         pvs = BlockDev.lvm_pvs()
-        self.assertTrue(len(pvs) > orig_len)
+        self.assertGreater(len(pvs), orig_len)
         self.assertTrue(any(info.pv_name == self.loop_dev for info in pvs))
 
         info = BlockDev.lvm_pvinfo(self.loop_dev)
@@ -574,7 +632,7 @@ class LvmTestVGinfo(LvmPVVGTestCase):
         self.assertEqual(info.name, "testVG")
         self.assertTrue(info.uuid)
         self.assertEqual(info.pv_count, 2)
-        self.assertTrue(info.size < 2 * 1024**3)
+        self.assertLess(info.size, 2 * 1024**3)
         self.assertEqual(info.free, info.size)
         self.assertEqual(info.extent_size, 4 * 1024**2)
 
@@ -592,7 +650,7 @@ class LvmTestVGs(LvmPVVGTestCase):
         self.assertTrue(succ)
 
         vgs = BlockDev.lvm_vgs()
-        self.assertTrue(len(vgs) > orig_len)
+        self.assertGreater(len(vgs), orig_len)
         self.assertTrue(any(info.name == "testVG" for info in vgs))
 
         info = BlockDev.lvm_vginfo("testVG")
@@ -613,6 +671,109 @@ class LvmTestVGs(LvmPVVGTestCase):
         succ = BlockDev.lvm_pvremove(self.loop_dev, None)
         self.assertTrue(succ)
 
+class LvmTestVGLocking(LvmPVVGTestCase):
+    @tag_test(TestTags.UNSAFE)
+    def test_vglock_stop_start(self):
+        """Verify that it is possible to start and stop locking on a VG"""
+
+        # better not do anything if lvmlockd is running, shared VGs have
+        # a tendency to wreak havoc on your system if you look at them wrong
+        ret, _out, _err = run_command("systemctl is-active lvmlockd")
+        if ret == 0:
+            self.skipTest("lvmlockd is running, skipping")
+
+        _ret, out, _err = run_command("lvm config 'global/use_lvmlockd'")
+        if "use_lvmlockd=0" not in out:
+            self.skipTest("lvmlockd is enabled, skipping")
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev2, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev, self.loop_dev2], 0, None)
+        self.assertTrue(succ)
+
+        # this actually doesn't "test" anything, the commands will just say lvmlockd is not
+        # running and return 0, but that's good enough for us
+        succ = BlockDev.lvm_vglock_start("testVG")
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vglock_stop("testVG")
+        self.assertTrue(succ)
+
+class LvmTestPVTags(LvmPVVGTestCase):
+    def test_pvtags(self):
+        """Verify that it's possible to set and get info about PV tags"""
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        # only pvs in a vg can be tagged so we need a vg here
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev], 0, None)
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_pvinfo(self.loop_dev)
+        self.assertTrue(info)
+        self.assertFalse(info.pv_tags)
+
+        succ = BlockDev.lvm_add_pv_tags(self.loop_dev, ["a", "b", "c"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_pvinfo(self.loop_dev)
+        self.assertTrue(info)
+        self.assertEqual(info.pv_tags, ["a", "b", "c"])
+
+        succ = BlockDev.lvm_delete_pv_tags(self.loop_dev, ["a", "b"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_pvinfo(self.loop_dev)
+        self.assertTrue(info)
+        self.assertEqual(info.pv_tags, ["c"])
+
+        succ = BlockDev.lvm_add_pv_tags(self.loop_dev, ["e"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_pvinfo(self.loop_dev)
+        self.assertTrue(info)
+        self.assertEqual(info.pv_tags, ["c", "e"])
+
+class LvmTestVGTags(LvmPVVGTestCase):
+    def test_vgtags(self):
+        """Verify that it's possible to set and get info about VG tags"""
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev], 0, None)
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_vginfo("testVG")
+        self.assertTrue(info)
+        self.assertFalse(info.vg_tags)
+
+        succ = BlockDev.lvm_add_vg_tags("testVG", ["a", "b", "c"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_vginfo("testVG")
+        self.assertTrue(info)
+        self.assertEqual(info.vg_tags, ["a", "b", "c"])
+
+        succ = BlockDev.lvm_delete_vg_tags("testVG", ["a", "b"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_vginfo("testVG")
+        self.assertTrue(info)
+        self.assertEqual(info.vg_tags, ["c"])
+
+        succ = BlockDev.lvm_add_vg_tags("testVG", ["e"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_vginfo("testVG")
+        self.assertTrue(info)
+        self.assertEqual(info.vg_tags, ["c", "e"])
+
 class LvmPVVGLVTestCase(LvmPVVGTestCase):
     def _clean_up(self):
         try:
@@ -649,7 +810,7 @@ class LvmTestLVcreateRemove(LvmPVVGLVTestCase):
         self.assertTrue(succ)
 
         # not enough space (only one PV)
-        with six.assertRaisesRegex(self, GLib.GError, "Insufficient free space"):
+        with self.assertRaisesRegex(GLib.GError, "Insufficient free space"):
             succ = BlockDev.lvm_lvcreate("testVG", "testLV", 1048 * 1024**2, None, [self.loop_dev], None)
 
         # enough space (two PVs)
@@ -672,12 +833,136 @@ class LvmTestLVcreateRemove(LvmPVVGLVTestCase):
         with self.assertRaises(GLib.GError):
             BlockDev.lvm_lvremove("testVG", "testLV", True, None)
 
+class LvmTestPartialLVs(LvmPVVGLVTestCase):
+    # the mirror halves are actually written to during sync-up and the
+    # default sparse_size of 1Gig is too much for a regular /tmp, so
+    # let's use smaller ones here.
+    #
+    _sparse_size = 20*1024**2
+
+    @tag_test(TestTags.CORE)
+    def test_lvpartial(self):
+        """Verify that missing PVs are detected and can be dealt with"""
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev2, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev3, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev, self.loop_dev2, self.loop_dev3], 0, None)
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_pvinfo(self.loop_dev2)
+        self.assertTrue(info)
+        self.assertFalse(info.missing)
+        self.assertEqual(info.vg_name, "testVG")
+        loop_dev2_pv_uuid = info.pv_uuid
+
+        # Create a mirrored LV on the first two PVs
+        with wait_for_sync("testVG", "testLV"):
+            succ = BlockDev.lvm_lvcreate("testVG", "testLV", 5 * 1024**2, "raid1",
+                                         [self.loop_dev, self.loop_dev2], None)
+            self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertTrue(info)
+        self.assertEqual(info.attr[8], "-")
+
+        # Check that lvs_tree returns the expected structure
+
+        def assert_lv_subs(info, segtype, len_segs, len_data, len_metadata):
+            self.assertTrue(info)
+            self.assertEqual(info.segtype, segtype)
+            self.assertEqual(len(info.segs), len_segs)
+            self.assertEqual(len(info.data_lvs), len_data)
+            self.assertEqual(len(info.metadata_lvs), len_metadata)
+
+        def assert_lv_single_pv(info, pv):
+            if pv:
+                assert_lv_subs(info, "linear", 1, 0, 0)
+                self.assertEqual(info.segs[0].pvdev, pv)
+            else:
+                assert_lv_subs(info, "linear", 0, 0, 0)
+
+        def assert_raid1_structure(pv1, pv2):
+            lvs = { lv.lv_name: lv for lv in BlockDev.lvm_lvs_tree("testVG") }
+            info = lvs["testLV"]
+            assert_lv_subs(info, "raid1", 0, 2, 2)
+            assert_lv_single_pv(lvs["["+info.data_lvs[0]+"]"], pv1)
+            assert_lv_single_pv(lvs["["+info.data_lvs[1]+"]"], pv2)
+            assert_lv_single_pv(lvs["["+info.metadata_lvs[0]+"]"], pv1)
+            assert_lv_single_pv(lvs["["+info.metadata_lvs[1]+"]"], pv2)
+
+        assert_raid1_structure(self.loop_dev, self.loop_dev2)
+
+        # Disconnect the second PV, this should cause it to be flagged
+        # as missing, and testLV to be reported as "partial".
+        delete_lio_device(self.loop_dev2)
+
+        pvs = BlockDev.lvm_pvs()
+        found = False
+        for pv in pvs:
+            if pv.pv_uuid == loop_dev2_pv_uuid:
+                found = True
+                self.assertTrue(pv.missing)
+                self.assertEqual(pv.vg_name, "testVG")
+        self.assertTrue(found)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertTrue(info)
+        self.assertEqual(info.attr[8], "p")
+
+        # lvs_tree should report the second stripe to be missing
+        assert_raid1_structure(self.loop_dev, None)
+
+        # remove records of missing PVs
+        succ = BlockDev.lvm_vgreduce("testVG", None, None)
+        self.assertTrue(succ)
+
+        pvs = BlockDev.lvm_pvs()
+        found = False
+        for pv in pvs:
+            if pv.pv_uuid == loop_dev2_pv_uuid:
+                found = True
+        self.assertFalse(found)
+
+        # lvs_tree should still report the second stripe to be missing
+        assert_raid1_structure(self.loop_dev, None)
+
+        # repair testLV with the third PV
+        with wait_for_sync("testVG", "testLV"):
+            succ = BlockDev.lvm_lvrepair("testVG", "testLV", [self.loop_dev3])
+            self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertEqual(info.attr[8], "-")
+
+        assert_raid1_structure(self.loop_dev, self.loop_dev3)
+
 class LvmTestLVcreateWithExtra(LvmPVVGLVTestCase):
     def __init__(self, *args, **kwargs):
         LvmPVVGLVTestCase.__init__(self, *args, **kwargs)
         self.log = ""
         self.ignore_log = True
 
+    @classmethod
+    def setUpClass(cls):
+        # we are checking for info log messages and default level is warning
+        BlockDev.utils_set_log_level(BlockDev.UTILS_LOG_INFO)
+
+        super(LvmTestLVcreateWithExtra, cls).setUpClass()
+
+    @classmethod
+    def tearDownClass(cls):
+        # reset back to default
+        BlockDev.utils_set_log_level(BlockDev.UTILS_LOG_WARNING)
+
+        super(LvmTestLVcreateWithExtra, cls).tearDownClass()
+
     def my_log_func(self, level, msg):
         if self.ignore_log:
             return
@@ -788,15 +1073,15 @@ class LvmTestLVactivateDeactivate(LvmPVVGLVTestCase):
         self.assertTrue(succ)
 
         with self.assertRaises(GLib.GError):
-            BlockDev.lvm_lvactivate("nonexistingVG", "testLV", True, None)
+            BlockDev.lvm_lvactivate("nonexistingVG", "testLV", True)
 
         with self.assertRaises(GLib.GError):
-            BlockDev.lvm_lvactivate("testVG", "nonexistingLV", True, None)
+            BlockDev.lvm_lvactivate("testVG", "nonexistingLV", True)
 
         with self.assertRaises(GLib.GError):
-            BlockDev.lvm_lvactivate("nonexistingVG", "nonexistingLV", True, None)
+            BlockDev.lvm_lvactivate("nonexistingVG", "nonexistingLV", True)
 
-        succ = BlockDev.lvm_lvactivate("testVG", "testLV", True, None)
+        succ = BlockDev.lvm_lvactivate("testVG", "testLV", True)
         self.assertTrue(succ)
 
         with self.assertRaises(GLib.GError):
@@ -811,7 +1096,15 @@ class LvmTestLVactivateDeactivate(LvmPVVGLVTestCase):
         succ = BlockDev.lvm_lvdeactivate("testVG", "testLV", None)
         self.assertTrue(succ)
 
-        succ = BlockDev.lvm_lvactivate("testVG", "testLV", True, None)
+        succ = BlockDev.lvm_lvactivate("testVG", "testLV", True)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_lvdeactivate("testVG", "testLV", None)
+        self.assertTrue(succ)
+
+        # try activating in shared mode, unfortunately no way to check whether it really
+        # works or not
+        succ = BlockDev.lvm_lvactivate("testVG", "testLV", True, True)
         self.assertTrue(succ)
 
         succ = BlockDev.lvm_lvdeactivate("testVG", "testLV", None)
@@ -858,7 +1151,7 @@ class LvmTestLVresize(LvmPVVGLVTestCase):
         succ = BlockDev.lvm_lvdeactivate("testVG", "testLV", None)
         self.assertTrue(succ)
 
-         # try to shrink when deactivated
+        # try to shrink when deactivated
         succ = BlockDev.lvm_lvresize("testVG", "testLV", 400 * 1024**2, None)
         self.assertTrue(succ)
 
@@ -966,7 +1259,7 @@ class LvmTestLVs(LvmPVVGLVTestCase):
         self.assertTrue(succ)
 
         lvs = BlockDev.lvm_lvs(None)
-        self.assertTrue(len(lvs) > orig_len)
+        self.assertGreater(len(lvs), orig_len)
         self.assertTrue(any(info.lv_name == "testLV" and info.vg_name == "testVG" for info in lvs))
 
         info = BlockDev.lvm_lvinfo("testVG", "testLV")
@@ -1002,6 +1295,12 @@ class LvmTestLVsMultiSegment(LvmPVVGLVTestCase):
         self.assertEqual(len(lvs), 1)
         self.assertListEqual([lv.lv_name for lv in lvs], ["testLV"])
 
+        # the LV will have a single segment on loop_dev
+        info = BlockDev.lvm_lvinfo_tree("testVG", "testLV")
+        self.assertEqual(info.segtype, "linear")
+        self.assertEqual(len(info.segs), 1)
+        self.assertEqual(info.segs[0].pvdev, self.loop_dev)
+
         # add second LV
         succ = BlockDev.lvm_lvcreate("testVG", "testLV2", 10 * 1024**2)
         self.assertTrue(succ)
@@ -1014,6 +1313,13 @@ class LvmTestLVsMultiSegment(LvmPVVGLVTestCase):
         succ = BlockDev.lvm_lvresize("testVG", "testLV", 20 * 1024**2, None)
         self.assertTrue(succ)
 
+        info = BlockDev.lvm_lvinfo_tree("testVG", "testLV")
+        self.assertEqual(info.segtype, "linear")
+        self.assertEqual(len(info.segs), 2)
+        self.assertEqual(info.segs[0].pvdev, self.loop_dev)
+        self.assertEqual(info.segs[1].pvdev, self.loop_dev)
+        self.assertNotEqual(info.segs[0].pv_start_pe, info.segs[1].pv_start_pe)
+
         lvs = BlockDev.lvm_lvs("testVG")
         self.assertEqual(len(lvs), 2)
         self.assertListEqual([lv.lv_name for lv in lvs], ["testLV", "testLV2"])
@@ -1111,26 +1417,22 @@ class LvmTestDataMetadataLV(LvmPVVGthpoolTestCase):
         succ = BlockDev.lvm_thpoolcreate("testVG", "testPool", 512 * 1024**2, 4 * 1024**2, 512 * 1024, "thin-performance", None)
         self.assertTrue(succ)
 
-        name = BlockDev.lvm_data_lv_name("testVG", "testPool")
         lvi = BlockDev.lvm_lvinfo("testVG", "testPool")
-        self.assertTrue(name)
-        self.assertTrue(name.startswith("testPool"))
-        self.assertIn("_tdata", name)
-        self.assertEqual(name, lvi.data_lv)
+        self.assertTrue(lvi.data_lv)
+        self.assertTrue(lvi.data_lv.startswith("testPool"))
+        self.assertIn("_tdata", lvi.data_lv)
 
-        info = BlockDev.lvm_lvinfo("testVG", name)
+        info = BlockDev.lvm_lvinfo("testVG", lvi.data_lv)
         self.assertTrue(info.attr.startswith("T"))
         self.assertIn("private", info.roles.split(","))
         self.assertIn("data", info.roles.split(","))
 
-        name = BlockDev.lvm_metadata_lv_name("testVG", "testPool")
         lvi = BlockDev.lvm_lvinfo("testVG", "testPool")
-        self.assertTrue(name)
-        self.assertTrue(name.startswith("testPool"))
-        self.assertIn("_tmeta", name)
-        self.assertEqual(name, lvi.metadata_lv)
+        self.assertTrue(lvi.metadata_lv)
+        self.assertTrue(lvi.metadata_lv.startswith("testPool"))
+        self.assertIn("_tmeta", lvi.metadata_lv)
 
-        info = BlockDev.lvm_lvinfo("testVG", name)
+        info = BlockDev.lvm_lvinfo("testVG", lvi.metadata_lv)
         self.assertTrue(info.attr.startswith("e"))
         self.assertIn("private", info.roles.split(","))
         self.assertIn("metadata", info.roles.split(","))
@@ -1226,8 +1528,11 @@ class LvmPVVGLVcachePoolTestCase(LvmPVVGLVTestCase):
             pass
 
         # lets help udev with removing stale symlinks
-        if not BlockDev.lvm_lvs("testVG") and os.path.exists("/dev/testVG/testCache_meta"):
-            shutil.rmtree("/dev/testVG", ignore_errors=True)
+        try:
+            if not BlockDev.lvm_lvs("testVG") and os.path.exists("/dev/testVG/testCache_meta"):
+                shutil.rmtree("/dev/testVG", ignore_errors=True)
+        except:
+            pass
 
         LvmPVVGLVTestCase._clean_up(self)
 
@@ -1436,58 +1741,165 @@ class LvmPVVGcachedThpoolstatsTestCase(LvmPVVGLVTestCase):
         self.assertEqual(stats.md_size, 8 * 1024**2)
         self.assertEqual(stats.mode, BlockDev.LVMCacheMode.WRITETHROUGH)
 
-class LVMUnloadTest(LVMTestCase):
-    def setUp(self):
-        # make sure the library is initialized with all plugins loaded for other
-        # tests
-        self.addCleanup(BlockDev.reinit, self.requested_plugins, True, None)
 
-    @tag_test(TestTags.NOSTORAGE)
-    def test_check_low_version(self):
-        """Verify that checking the minimum LVM version works as expected"""
+class LvmPVVGLVWritecacheAttachDetachTestCase(LvmPVVGLVcachePoolTestCase):
+    @tag_test(TestTags.SLOW)
+    def test_writecache_attach_detach(self):
+        """Verify that is it possible to attach and detach a writecache LV"""
 
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
+        lvm_version = self._get_lvm_version()
+        if lvm_version < Version("2.03.02"):
+            self.skipTest("LVM writecache support not available")
 
-        with fake_utils("tests/lvm_low_version/"):
-            # too low version of LVM available, the LVM plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
+        lvm_segtypes = self._get_lvm_segtypes()
+        if "writecache" not in lvm_segtypes:
+            self.skipTest("LVM writecache support not available")
 
-            self.assertNotIn("lvm", BlockDev.get_available_plugin_names())
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
 
-        # load the plugins back
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-        self.assertIn("lvm", BlockDev.get_available_plugin_names())
+        succ = BlockDev.lvm_pvcreate(self.loop_dev2, 0, 0, None)
+        self.assertTrue(succ)
 
-    @tag_test(TestTags.NOSTORAGE)
-    def test_check_no_lvm(self):
-        """Verify that checking lvm tool availability works as expected"""
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev, self.loop_dev2], 0, None)
+        self.assertTrue(succ)
 
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
+        succ = BlockDev.lvm_lvcreate("testVG", "testCache", 512 * 1024**2, None, [self.loop_dev2], None)
+        self.assertTrue(succ)
 
-        with fake_path(all_but="lvm"):
-            # no lvm tool available, the LVM plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
+        succ = BlockDev.lvm_lvcreate("testVG", "testLV", 512 * 1024**2, None, [self.loop_dev], None)
+        self.assertTrue(succ)
 
-            self.assertNotIn("lvm", BlockDev.get_available_plugin_names())
+        succ = BlockDev.lvm_writecache_attach("testVG", "testLV", "testCache", None)
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertIsNotNone(info)
+        self.assertEqual(info.segtype, "writecache")
+
+        # detach and destroy (the last arg)
+        succ = BlockDev.lvm_writecache_detach("testVG", "testLV", True, None)
+        self.assertTrue(succ)
+
+        # once more and do not destroy this time
+        succ = BlockDev.lvm_lvcreate("testVG", "testCache", 512 * 1024**2, None, [self.loop_dev2], None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_writecache_attach("testVG", "testLV", "testCache", None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_writecache_detach("testVG", "testLV", False, None)
+        self.assertTrue(succ)
+
+        lvs = BlockDev.lvm_lvs("testVG")
+        self.assertTrue(any(info.lv_name == "testCache" for info in lvs))
+
+class LvmPVVGWritecachedLVTestCase(LvmPVVGLVTestCase):
+    @tag_test(TestTags.SLOW)
+    def test_create_cached_lv(self):
+        """Verify that it is possible to create a writecached LV in a single step"""
+
+        lvm_version = self._get_lvm_version()
+        if lvm_version < Version("2.03.02"):
+            self.skipTest("LVM writecache support not available")
+
+        lvm_segtypes = self._get_lvm_segtypes()
+        if "writecache" not in lvm_segtypes:
+            self.skipTest("LVM writecache support not available")
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev2, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev, self.loop_dev2], 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_writecache_create_cached_lv("testVG", "testLV", 512 * 1024**2, 256 * 1024**2,
+                                                        [self.loop_dev], [self.loop_dev2])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertIsNotNone(info)
+        self.assertEqual(info.segtype, "writecache")
+
+class LvmVGExportedTestCase(LvmPVVGLVTestCase):
+
+    def _clean_up(self):
+        run_command("vgimport testVG")
+
+        LvmPVVGLVTestCase._clean_up(self)
+
+    @tag_test(TestTags.SLOW)
+    def test_exported_vg(self):
+        """Verify that info has correct information about exported VGs"""
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev2, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev, self.loop_dev2], 0, None)
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_vginfo("testVG")
+        self.assertTrue(info)
+        self.assertFalse(info.exported)
+
+        ret, out, err = run_command("vgexport testVG")
+        if ret != 0:
+            self.fail("Failed to export VG:\n%s %s" % (out, err))
+
+        info = BlockDev.lvm_vginfo("testVG")
+        self.assertTrue(info)
+        self.assertTrue(info.exported)
+
+
+        self.assertTrue(succ)
+
+class LvmTestLVTags(LvmPVVGLVTestCase):
+    def test_vgtags(self):
+        """Verify that it's possible to set and get info about LV tags"""
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev, 0, 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_vgcreate("testVG", [self.loop_dev], 0, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_lvcreate("testVG", "testLV", 512 * 1024**2, None, [self.loop_dev], None)
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertTrue(info)
+        self.assertFalse(info.lv_tags)
+
+        succ = BlockDev.lvm_add_lv_tags("testVG", "testLV", ["a", "b", "c"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertTrue(info)
+        self.assertEqual(info.lv_tags, ["a", "b", "c"])
+
+        succ = BlockDev.lvm_delete_lv_tags("testVG", "testLV", ["a", "b"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertTrue(info)
+        self.assertEqual(info.lv_tags, ["c"])
+
+        succ = BlockDev.lvm_add_lv_tags("testVG", "testLV", ["e"])
+        self.assertTrue(succ)
+
+        info = BlockDev.lvm_lvinfo("testVG", "testLV")
+        self.assertTrue(info)
+        self.assertEqual(info.lv_tags, ["c", "e"])
 
-        # load the plugins back
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-        self.assertIn("lvm", BlockDev.get_available_plugin_names())
 
 class LVMTechTest(LVMTestCase):
 
-    def setUp(self):
-        # set init checks to false -- we want runtime checks for this
-        BlockDev.switch_init_checks(False)
-
-        # set everything back and reinit just to be sure
-        self.addCleanup(BlockDev.switch_init_checks, True)
-        self.addCleanup(BlockDev.reinit, self.requested_plugins, True, None)
-
     @tag_test(TestTags.NOSTORAGE)
     def test_tech_available(self):
         """Verify that checking lvm tool availability by technology works as expected"""
@@ -1500,7 +1912,7 @@ class LVMTechTest(LVMTestCase):
                 BlockDev.lvm_is_tech_avail(BlockDev.LVMTech.BASIC, BlockDev.LVMTechMode.CREATE)
 
         # only query is support with calcs
-        with six.assertRaisesRegex(self, GLib.GError, "Only 'query' supported for thin calculations"):
+        with self.assertRaisesRegex(GLib.GError, "Only 'query' supported for thin calculations"):
             BlockDev.lvm_is_tech_avail(BlockDev.LVMTech.THIN_CALCS, BlockDev.LVMTechMode.CREATE)
 
         # lvm is available, should pass
@@ -1617,6 +2029,20 @@ class LVMVDOTest(LVMTestCase):
         self.assertTrue(vdo_info.deduplication)
         self.assertEqual(BlockDev.lvm_get_vdo_write_policy_str(vdo_info.write_policy), "sync")
 
+    @tag_test(TestTags.SLOW)
+    def test_vdo_pool_create_noname(self):
+        succ = BlockDev.lvm_vdo_pool_create("testVDOVG", "vdoLV", None, 7 * 1024**3, 35 * 1024**3)
+        self.assertTrue(succ)
+
+        lv_info = BlockDev.lvm_lvinfo("testVDOVG", "vdoLV")
+        self.assertIsNotNone(lv_info)
+        self.assertEqual(lv_info.segtype, "vdo")
+
+        pool_name = BlockDev.lvm_vdolvpoolname("testVDOVG", "vdoLV")
+        self.assertEqual(lv_info.pool_lv, pool_name)
+        pool_info = BlockDev.lvm_lvinfo("testVDOVG", pool_name)
+        self.assertEqual(pool_info.segtype, "vdo-pool")
+
     @tag_test(TestTags.SLOW)
     def test_resize(self):
         succ = BlockDev.lvm_vdo_pool_create("testVDOVG", "vdoLV", "vdoPool", 5 * 1024**3, 10 * 1024**3)
@@ -1638,7 +2064,7 @@ class LVMVDOTest(LVMTestCase):
         self.assertEqual(lv_info.size, 35 * 1024**3)
 
     @tag_test(TestTags.SLOW)
-    def test_enabla_disable_compression(self):
+    def test_enable_disable_compression(self):
         succ = BlockDev.lvm_vdo_pool_create("testVDOVG", "vdoLV", "vdoPool", 7 * 1024**3, 35 * 1024**3)
         self.assertTrue(succ)
 
@@ -1737,6 +2163,64 @@ class LVMVDOTest(LVMTestCase):
         self.assertIn("writeAmplificationRatio", full_stats.keys())
 
 
+class LvmTestDevicesFile(LvmPVonlyTestCase):
+    devicefile = "bd_lvm_test.devices"
+
+    @classmethod
+    def tearDownClass(cls):
+        shutil.rmtree("/etc/lvm/devices/" + cls.devicefile, ignore_errors=True)
+
+        super(LvmTestDevicesFile, cls).tearDownClass()
+
+    def test_devices_add_delete(self):
+        if not self.devices_avail:
+            self.skipTest("skipping LVM devices file test: not supported")
+
+        self.addCleanup(BlockDev.lvm_set_global_config, None)
+
+        # force-enable the feature, it might be disabled by default
+        succ = BlockDev.lvm_set_global_config("devices { use_devicesfile=1 }")
+        self.assertTrue(succ)
+
+        succ = BlockDev.lvm_pvcreate(self.loop_dev)
+        self.assertTrue(succ)
+
+        with self.assertRaises(GLib.GError):
+            BlockDev.lvm_devices_add("/non/existing/device", self.devicefile)
+
+        with self.assertRaises(GLib.GError):
+            BlockDev.lvm_devices_delete(self.loop_dev, self.devicefile)
+
+        succ = BlockDev.lvm_devices_add(self.loop_dev, self.devicefile)
+        self.assertTrue(succ)
+
+        dfile = read_file("/etc/lvm/devices/" + self.devicefile)
+        self.assertIn(self.loop_dev, dfile)
+
+        succ = BlockDev.lvm_devices_delete(self.loop_dev, self.devicefile)
+        self.assertTrue(succ)
+
+        dfile = read_file("/etc/lvm/devices/" + self.devicefile)
+        self.assertNotIn(self.loop_dev, dfile)
+
+        BlockDev.lvm_set_global_config(None)
+
+    def test_devices_enabled(self):
+        if not self.devices_avail:
+            self.skipTest("skipping LVM devices file test: not supported")
+
+        self.addCleanup(BlockDev.lvm_set_global_config, None)
+
+        # checking if the feature is enabled or disabled is hard so lets just disable
+        # the devices file using the global config and check lvm_devices_add fails
+        # with the correct exception message
+        succ = BlockDev.lvm_set_global_config("devices { use_devicesfile=0 }")
+        self.assertTrue(succ)
+
+        with self.assertRaisesRegex(GLib.GError, "LVM devices file not enabled."):
+            BlockDev.lvm_devices_add("", self.devicefile)
+
+
 class LvmConfigTestPvremove(LvmPVonlyTestCase):
 
     @tag_test(TestTags.REGRESSION)
@@ -1745,7 +2229,5 @@ class LvmConfigTestPvremove(LvmPVonlyTestCase):
         self.assertTrue(succ)
 
         BlockDev.lvm_set_global_config("")
-
-        ea = BlockDev.ExtraArg.new("--config", "devices {use_devicesfile = 0}")
-        succ = BlockDev.lvm_pvremove(self.loop_dev, extra=[ea])
+        succ = BlockDev.lvm_pvremove(self.loop_dev)
         self.assertTrue(succ)
