@@ -1,5 +1,6 @@
 import unittest
 import os
+import resource
 import overrides_hack
 
 from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, fake_utils, fake_path, run_command, run, TestTags, tag_test
@@ -8,6 +9,7 @@ from gi.repository import BlockDev, GLib
 
 class SwapTest(unittest.TestCase):
     requested_plugins = BlockDev.plugin_specs_from_names(("swap",))
+    dev_size = 1024**3
 
     @classmethod
     def setUpClass(cls):
@@ -16,10 +18,17 @@ class SwapTest(unittest.TestCase):
         else:
             BlockDev.reinit(cls.requested_plugins, True, None)
 
+class SwapPluginVersionTestCase(SwapTest):
+    @tag_test(TestTags.NOSTORAGE)
+    def test_plugin_version(self):
+       self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.SWAP), "libbd_swap.so.3")
+
 class SwapTestCase(SwapTest):
+    test_uuid = "4d7086c4-a4d3-432f-819e-73da03870df9"
+
     def setUp(self):
         self.addCleanup(self._clean_up)
-        self.dev_file = create_sparse_tempfile("swap_test", 1024**3)
+        self.dev_file = create_sparse_tempfile("swap_test", self.dev_size)
         try:
             self.loop_dev = create_lio_device(self.dev_file)
         except RuntimeError as e:
@@ -73,6 +82,12 @@ class SwapTestCase(SwapTest):
         _ret, out, _err = run_command("blkid -ovalue -sLABEL -p %s" % self.loop_dev)
         self.assertEqual(out, "BlockDevSwap")
 
+        succ = BlockDev.swap_set_uuid(self.loop_dev, uuid=self.test_uuid)
+        self.assertTrue(succ)
+
+        _ret, out, _err = run_command("blkid -ovalue -sUUID -p %s" % self.loop_dev)
+        self.assertEqual(out, self.test_uuid)
+
         succ = BlockDev.swap_swapon(self.loop_dev, -1)
         self.assertTrue(succ)
 
@@ -98,11 +113,27 @@ class SwapTestCase(SwapTest):
         _ret, out, _err = run_command("blkid -ovalue -sLABEL -p %s" % self.loop_dev)
         self.assertEqual(out, "BlockDevSwap")
 
+    def test_mkswap_with_uuid(self):
+        """Verify that mkswap with uuid works as expected"""
+
+        succ = BlockDev.swap_mkswap(self.loop_dev, uuid=self.test_uuid)
+        self.assertTrue(succ)
+
+        _ret, out, _err = run_command("blkid -ovalue -sUUID -p %s" % self.loop_dev)
+        self.assertEqual(out, self.test_uuid)
+
     def test_swapon_pagesize(self):
         """Verify that activating swap with different pagesize fails"""
 
-        # create swap with 64k pagesize
-        ret, out, err = run_command("mkswap --pagesize 65536 %s" % self.loop_dev)
+        # pick some wrong page size: 8k on 64k and 64k everywhere else
+        pagesize = resource.getpagesize()
+        if pagesize == 65536:
+            wrong_pagesize = 8192
+        else:
+            wrong_pagesize = 65536
+
+        # create swap with "wrong" pagesize
+        ret, out, err = run_command("mkswap --pagesize %s %s" % (wrong_pagesize, self.loop_dev))
         if ret != 0:
             self.fail("Failed to prepare swap for pagesize test: %s %s" % (out, err))
 
@@ -110,6 +141,18 @@ class SwapTestCase(SwapTest):
         with self.assertRaises(BlockDev.SwapPagesizeError):
             BlockDev.swap.swapon(self.loop_dev)
 
+    def test_swapon_wrong_size(self):
+        """Verify that activating swap with a wrong size fails with expected exception"""
+
+        # create swap bigger than the device (twice as big in 1024 sectors)
+        ret, out, err = run_command("mkswap -f %s %d" % (self.loop_dev, (self.dev_size * 2) / 1024))
+        if ret != 0:
+            self.fail("Failed to prepare swap for wrong size test: %s %s" % (out, err))
+
+        # activation should fail because swap is bigger than the underlying device
+        with self.assertRaises(BlockDev.SwapActivateError):
+            BlockDev.swap.swapon(self.loop_dev)
+
     def _remove_map(self, map_name):
         run("dmsetup remove -f %s" % map_name)
 
@@ -141,83 +184,34 @@ class SwapTestCase(SwapTest):
         self.assertFalse(on)
 
 
-class SwapUnloadTest(SwapTest):
-    def setUp(self):
-        # make sure the library is initialized with all plugins loaded for other
-        # tests
-        self.addCleanup(BlockDev.reinit, self.requested_plugins, True, None)
-
+class SwapDepsTest(SwapTest):
     @tag_test(TestTags.NOSTORAGE)
-    def test_check_low_version(self):
-        """Verify that checking the minimum swap utils versions works as expected"""
+    def test_missing_dependencies(self):
+        """Verify that checking for technology support works as expected"""
 
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
-
-        with fake_utils("tests/swap_low_version/"):
-            # too low version of mkswap available, the swap plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
-
-            self.assertNotIn("swap", BlockDev.get_available_plugin_names())
-
-        # load the plugins back
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-        self.assertIn("swap", BlockDev.get_available_plugin_names())
-
-    @tag_test(TestTags.NOSTORAGE)
-    def test_check_no_mkswap(self):
-        """Verify that checking mkswap and swaplabel tools availability
-           works as expected
-        """
-
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
+        with fake_utils("tests/fake_utils/swap_low_version/"):
+            # too low version of mkswap available
+            with self.assertRaisesRegexp(GLib.GError, "Too low version of mkswap"):
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP, BlockDev.SwapTechMode.CREATE)
 
         with fake_path(all_but="mkswap"):
-            # no mkswap available, the swap plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
-
-            self.assertNotIn("swap", BlockDev.get_available_plugin_names())
+            # no mkswap available
+            with self.assertRaisesRegexp(GLib.GError, "The 'mkswap' utility is not available"):
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP, BlockDev.SwapTechMode.CREATE)
 
         with fake_path(all_but="swaplabel"):
-            # no swaplabel available, the swap plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
-
-            self.assertNotIn("swap", BlockDev.get_available_plugin_names())
-
-        # load the plugins back
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-        self.assertIn("swap", BlockDev.get_available_plugin_names())
-
-    @tag_test(TestTags.NOSTORAGE)
-    def test_check_no_mkswap_runtime(self):
-        """Verify that runtime checking mkswap tool availability works as expected"""
-
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
-
-        # make sure the initial checks during plugin loading are skipped
-        BlockDev.switch_init_checks(False)
-        self.addCleanup(BlockDev.switch_init_checks, True)
+            # no swaplabel available
+            with self.assertRaisesRegexp(GLib.GError, "The 'swaplabel' utility is not available"):
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP, BlockDev.SwapTechMode.SET_LABEL)
 
         with fake_path(all_but="mkswap"):
-            # no mkswap available, but checks disabled, the swap plugin should load just fine
-            self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-            self.assertIn("swap", BlockDev.get_available_plugin_names())
-
             with self.assertRaisesRegexp(GLib.GError, "The 'mkswap' utility is not available"):
                 # the device shouldn't matter, the function should return an
                 # error before any other checks or actions
                 BlockDev.swap_mkswap("/dev/device", "LABEL", None)
 
+
 class SwapTechAvailable(SwapTest):
-    def setUp(self):
-        # set everything back and reinit just to be sure
-        self.addCleanup(BlockDev.switch_init_checks, True)
-        self.addCleanup(BlockDev.reinit, self.requested_plugins, True, None)
 
     @tag_test(TestTags.NOSTORAGE)
     def test_check_tech_available(self):
@@ -226,39 +220,47 @@ class SwapTechAvailable(SwapTest):
         """
 
         with fake_path(all_but="mkswap"):
-            BlockDev.switch_init_checks(False)
             BlockDev.reinit(self.requested_plugins, True, None)
 
             with self.assertRaises(GLib.GError):
                 # we have swaplabel but not mkswap, so this should fail
-                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP_TECH_SWAP,
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
                                             BlockDev.SwapTechMode.CREATE | BlockDev.SwapTechMode.SET_LABEL)
 
             with self.assertRaises(GLib.GError):
                 # we have swaplabel but not mkswap, so this should fail
-                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP_TECH_SWAP,
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
                                             BlockDev.SwapTechMode.CREATE)
 
             # only label checked -- should pass
-            succ = BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP_TECH_SWAP,
+            succ = BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
                                                BlockDev.SwapTechMode.SET_LABEL)
             self.assertTrue(succ)
 
+            # only UUID checked -- should pass
+            succ = BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
+                                               BlockDev.SwapTechMode.SET_UUID)
+            self.assertTrue(succ)
+
         with fake_path(all_but="swaplabel"):
-            BlockDev.switch_init_checks(False)
             BlockDev.reinit(self.requested_plugins, True, None)
 
             with self.assertRaises(GLib.GError):
                 # we have mkswap but not swaplabel, so this should fail
-                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP_TECH_SWAP,
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
                                             BlockDev.SwapTechMode.CREATE | BlockDev.SwapTechMode.SET_LABEL)
 
             with self.assertRaises(GLib.GError):
                 # we have mkswap but not swaplabel, so this should fail
-                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP_TECH_SWAP,
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
                                             BlockDev.SwapTechMode.SET_LABEL)
 
+            with self.assertRaises(GLib.GError):
+                # we have mkswap but not swaplabel, so this should fail
+                BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
+                                            BlockDev.SwapTechMode.SET_UUID)
+
             # only label checked -- should pass
-            succ = BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP_TECH_SWAP,
+            succ = BlockDev.swap_is_tech_avail(BlockDev.SwapTech.SWAP,
                                                BlockDev.SwapTechMode.CREATE)
             self.assertTrue(succ)
